$fa = 1;
$fs = 0.4;

// switch measurements
joycon_length = 100.8;
joycon_thickness = 14;
joycon_mount_width = 10;
front_curve_radius = 3.3;
back_curve_radius = 6.3;

// parameters
switch_mount_height = joycon_length;
joint_thickness = joycon_thickness-5;
joint_length = joycon_length-2*back_curve_radius;
joint_width = 5;
wall_thickness = 2.5;

// left joycon in print orientation
translate([-75,0,0])
rotate([0,-90,0])
left_joycon_receptical_with_joint();

// right joycon in print orientation
translate([-50,0,joycon_mount_width+joint_width])
rotate([0,90,0])
right_joycon_receptical_with_joint();

// screen mount in print orientation
rotate([0, -90, 0])
    screen_mount();

/*
// left joycon in assembled position
rotate([0, -90, 0])
translate([0,joycon_length+joycon_mount_width+joint_width+2*wall_thickness, 2.5])
rotate([0,0,-90])
    left_joycon_receptical_with_joint();

// right joycon in assembled position
rotate([0, -90, 0])
translate([0, -0.5, wall_thickness])
rotate([0,0,-90])
right_joycon_receptical_with_joint();
*/

module screen_mount() {
    difference() {
        translate([switch_mount_height, -joint_width-0.5, 0])
        rotate([0,0,90])
        switch_profile(
            joycon_length+2*(joint_width+0.5)+2*wall_thickness,
            switch_mount_height,
            joycon_thickness+2*wall_thickness,
            front_curve_radius,
            back_curve_radius);

        // cutout to fit switch
        translate([wall_thickness, wall_thickness, wall_thickness])
            switch_profile(
                switch_mount_height,
                joycon_length+1,
                joycon_thickness+1,
                front_curve_radius,
                back_curve_radius);
        
        //cutout to fit smaller radius corner near USB-C port
        translate([60+wall_thickness, (joycon_length+1)/2+wall_thickness, wall_thickness])
            switch_profile(
                switch_mount_height,
                (joycon_length+1)/2,
                joycon_thickness+1,
                front_curve_radius,
                front_curve_radius);
        
        //screen cutout
        translate([wall_thickness+14, wall_thickness+8, joycon_thickness+wall_thickness])
            cube([switch_mount_height, joycon_length-16, wall_thickness+2]);

        // exhaust vent cutout
        exhaust_length = 38.4;
        exhaust_width = 5;
        exhaust_from_bottom = 51; // (when in portrait orientation)
        exhaust_cut_height = wall_thickness+joycon_thickness-front_curve_radius;

        translate([
            exhaust_from_bottom+wall_thickness,
            wall_thickness/2,
            -1])

           cube([exhaust_length-exhaust_width, exhaust_width, exhaust_cut_height]);

        translate([
            exhaust_from_bottom+wall_thickness,
            wall_thickness/2+exhaust_width/2,
            -1])
 
            cylinder(r=exhaust_width/2, h=exhaust_cut_height);

        translate([
            exhaust_from_bottom+exhaust_length-exhaust_width+wall_thickness, 
            wall_thickness/2+exhaust_width/2,
            -1])
            
            cylinder(r=exhaust_width/2, h=exhaust_cut_height);
            
        //intake vent cutout
        vent_length = 43;
        vent_width = 5;
        vent_dist_from_bottom = 36; //(when in portrait orientation)
        vent_dist_from_left = 8.6; //(when in portraint orientation)
       
        translate([
            vent_width/2+vent_dist_from_bottom+wall_thickness,
            joycon_length-vent_dist_from_left,
            -0.5])
     
            cube([vent_length-vent_width, vent_width, wall_thickness+1]);
        
        translate([
            vent_width/2+vent_dist_from_bottom+wall_thickness,
            joycon_length-vent_dist_from_left+vent_width/2,
            -0.5]) 
 
            cylinder(r=vent_width/2, h=wall_thickness+1);
            
        translate([
            vent_dist_from_bottom+vent_length-vent_width/2+wall_thickness,
            joycon_length-vent_dist_from_left+vent_width/2,
            -0.5]) 

            cylinder(r=vent_width/2, h=wall_thickness+1);
            
        // lower speaker cutout
        speaker_dist_from_bottom = 17;
        speaker_dist_from_left = 2.4;
        speaker_length = 8.2;
        speaker_width = 2;
        
        translate([
            speaker_width/2+speaker_dist_from_bottom+wall_thickness,
            joycon_length-speaker_dist_from_left,
            joycon_thickness+wall_thickness-0.5])
 
            cube([speaker_length-speaker_width, speaker_width, wall_thickness+1]);
        
        translate([
            speaker_width/2+speaker_dist_from_bottom+wall_thickness,
            joycon_length-speaker_dist_from_left+speaker_width/2,
            joycon_thickness+wall_thickness-0.5])
            
            cylinder(r=speaker_width/2, h=wall_thickness+1);

        translate([
            speaker_dist_from_bottom+speaker_length-speaker_width/2+wall_thickness,
            joycon_length-speaker_dist_from_left+speaker_width/2,
            joycon_thickness+wall_thickness-0.5])
            
            cylinder(r=speaker_width/2, h=wall_thickness+1);
  
        //right joycon attachment
        translate(
            [-1,
            0,
            ((joycon_thickness+2*wall_thickness)-(joint_thickness+0.5))/2])
        rotate([0,0,-90])
        
            dovetail([
                1+joint_width+0.5,
                joint_length+(joycon_length-joint_length)/2+0.5,
                joint_thickness+0.5]);

        //bevel edges on right joycon side
        translate(
            [0,
            -joint_width-0.5,
            joycon_thickness+2*wall_thickness-front_curve_radius])
        curve_cutter(switch_mount_height, front_curve_radius);
        
        translate(
            [0,
            front_curve_radius-joint_width-0.5,
            0])
        rotate([90,0,0])
        curve_cutter(switch_mount_height, front_curve_radius);
        
        //left joycon attachment
        translate(
            [-1,
            joycon_length+2*wall_thickness,
            ((joycon_thickness+2*wall_thickness)-(joint_thickness+0.5))/2])
        mirror([0,1,0])
        rotate([0,0,-90])
            dovetail([
                1+joint_width+0.5,
                joint_length+(joycon_length-joint_length)/2+0.5,
                joint_thickness+0.5]);
            
        //bevel edges on left joycon side
        translate(
            [0,
            joycon_length+2*wall_thickness+joint_width+0.5,
            joycon_thickness+2*wall_thickness-front_curve_radius])
        mirror([0,1,0])
        curve_cutter(switch_mount_height, front_curve_radius);
        
        translate(
            [0,
            joycon_length+2*wall_thickness+joint_width+0.5-front_curve_radius,
            0])
        mirror([0,1,0])
        rotate([90,0,0])
        curve_cutter(switch_mount_height, front_curve_radius);
    }
}




module left_joycon_receptical_with_joint() {
    
    translate([joycon_mount_width + joint_width,0,0])
    mirror([1,0,0])
        right_joycon_receptical_with_joint();   
}

module right_joycon_receptical_with_joint() {


    translate([joint_width, 0, 0])
        right_joycon_receptical();
    
    translate(
        [0,
        (joycon_length-joint_length)/2,
        (joycon_thickness-joint_thickness)/2])

        dovetail([joint_width, joint_length, joint_thickness]);
}


module right_joycon_receptical() {
    
    difference() {
        switch_profile(
            joycon_mount_width, joycon_length, joycon_thickness, front_curve_radius, back_curve_radius);
        
        right_joycon_slot();
    }
}

module dovetail(v) {
    difference() {        
        x = v[0];
        y = v[1];
        z = v[2];

        cube([x, y, z]);
        
        translate([0,-2,z])
        rotate([0,10,0])
            cube([x*2, y+4, z]);
        
        rotate([0,-10,0])
        translate([0,-2,-z])
            cube([x*2, y+4, z]);
    }
}

module curve_cutter(width, radius) {

    difference() {
        translate([-1, -1, 0])
            cube([width+2, radius+1, radius+1]);
            
        translate([-2, radius, 0])
        rotate([0,90,0])
            cylinder(r=radius, h=width+4);
    }
}

module joycon_receptical_square(
    joycon_mount_width, joycon_length, joycon_thickness) {

    difference() {
        cube([joycon_mount_width, joycon_length, joycon_thickness]);
      
    }
}

module switch_profile(
    width,
    height,
    thickness,
    front_curve_radius,
    back_curve_radius)
{
    difference() {
        cube([width, height, thickness]);
        
        // cut curve into bottom front edge
        translate([0, 0, thickness-front_curve_radius])
            curve_cutter(width, front_curve_radius);
        
        // cut curve into top front edge
        translate([0, height-front_curve_radius, thickness])
        rotate([-90, 0, 0])
            curve_cutter(width, front_curve_radius);
        
        // cut curve into top back edge
        translate([0, height, back_curve_radius])
        rotate([-180, 0, 0])
            curve_cutter(width, back_curve_radius);

        // cut curve into bottom back edge
        translate([0, back_curve_radius, 0])
        rotate([90, 0, 0])
            curve_cutter(width, back_curve_radius);
    }
}

module right_joycon_slot() {
    //joycon slot measurements
    slot_width = 10;
    slot_depth = 3;
    slot_length = 95.5;
    track_thickness = 0.4;
    track_space = 7.9;
    lock_cutout_distance_from_slot_top = 4;
    lock_cutout_length = 2; // was 3
    lock_cutout_angled_length = 1;

    // cut the slot where the black part of the joycon goes
    translate(
        [joycon_mount_width-slot_depth-track_thickness,
        -1,
        (joycon_thickness-slot_width)/2])
    
            cube([slot_depth, slot_length+1, slot_width]);

    // cut out space to leave a track for the joycon to ride on
    // (The part that goes under the edge of the black part of the joycon)
    translate(
        [joycon_mount_width-slot_depth-track_thickness,
        -1,
        (joycon_thickness-track_space)/2])
    
        cube(
            [slot_depth+track_thickness+1,
            slot_length+1,
            track_space]);
    
     // cut a space out of the back track for the locking mechanism
     translate(
        [joycon_mount_width-track_thickness-0.2,
        slot_length - lock_cutout_distance_from_slot_top - lock_cutout_length,
        (joycon_thickness-slot_width)/2])
        
            cube(
                [track_thickness+0.4,
                lock_cutout_length,
                slot_width-track_space/2]);

     // cut an angle into the back track of the space for the angled part of the
     // locking mechanism
     translate(
        [joycon_mount_width-track_thickness-0.2,
        slot_length - lock_cutout_distance_from_slot_top - lock_cutout_length,
        (joycon_thickness-slot_width)/2])
     rotate([45,0,0])
            cube(
                [track_thickness+0.4,
                slot_width-track_space/2,
                slot_width-track_space/2]);
                
     // an angle into the bottom of the front joycon track
     translate(
        [joycon_mount_width-track_thickness-0.2,
        0,
        track_space-(joycon_thickness-slot_width)/2+slot_width-track_space/2+0.1])
     rotate([-90-10,0,0.0])
            cube(
                [track_thickness+0.4,
                slot_width-track_space/2,
                slot_width-track_space/2+2]); 
   
     // an angle into the bottom of the back joycon track
     translate(
        [joycon_mount_width-track_thickness-0.2,
        0,
        (joycon_thickness-slot_width)/2-0.3])
     rotate([10,0,0.0])
            cube(
                [track_thickness+0.4,
                slot_width-track_space/2+2,
                slot_width-track_space/2]);   
}